# -*- coding: utf-8 -*-
from collective.lineage.testing import LINEAGE_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.browserlayer.utils import registered_layers

try:
    # BBB
    import unittest2 as unittest
except ImportError:
    import unittest

PROJECTNAME = 'collective.lineage'


class InstallTestCase(unittest.TestCase):

    layer = LINEAGE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.qi = self.portal.portal_quickinstaller

    def test_installed(self):
        self.assertTrue(self.qi.isProductInstalled(PROJECTNAME))

    def test_addon_layer(self):
        layers = [l.getName() for l in registered_layers()]
        self.assertTrue('ILineageBrowserLayer' in layers,
                        'add-on layer was not installed')

    # TODO: test propertiestool and viewlets


class UninstallTest(unittest.TestCase):

    layer = LINEAGE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.qi = self.portal.portal_quickinstaller
        self.qi.uninstallProducts(products=[PROJECTNAME])

    def test_uninstalled(self):
        self.assertFalse(self.qi.isProductInstalled(PROJECTNAME))

    def test_addon_layer_removed(self):
        layers = [l.getName() for l in registered_layers()]
        self.assertFalse('ILineageBrowserLayer' in layers,
                         'add-on layer was not removed')



# -*- coding: utf-8 -*-
from collective.lineage.events import ChildSiteCreatedEvent
from collective.lineage.events import ChildSiteRemovedEvent
from collective.lineage.events import ChildSiteWillBeCreatedEvent
from collective.lineage.events import ChildSiteWillBeRemovedEvent
from collective.lineage.interfaces import IChildSite
from five.localsitemanager import make_objectmanager_site
from Products.Five.component import disableSite
from zope.component.interfaces import ISite
from zope.event import notify
from zope.interface import alsoProvides
from zope.interface import noLongerProvides


def enable_childsite(context):
    notify(ChildSiteWillBeCreatedEvent(context))

    # enable site
    if not ISite.providedBy(context):
        make_objectmanager_site(context)

    # provide IChildSite
    alsoProvides(context, IChildSite)

    context.reindexObject(idxs=('object_provides'))
    notify(ChildSiteCreatedEvent(context))


def disable_childsite(context):
    notify(ChildSiteWillBeRemovedEvent(context))

    # remove local site components
    disableSite(context)

    # remove IChildSite
    noLongerProvides(context, IChildSite)

    context.reindexObject(idxs=('object_provides'))
    notify(ChildSiteRemovedEvent(context))



# -*- coding: utf-8 -*-
from plone.app.layout.navigation.interfaces import INavigationRoot
from zope.component.interfaces import IPossibleSite
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface

_ = MessageFactory('collective.lineage')


class ILineageBrowserLayer(Interface):
    """Browser layer marker interface
    """


class IChildSite(INavigationRoot, IPossibleSite):
    """A marker interface for a Child Site. This is comprised of
    several other marker interfaces.

    INavigationRoot
      make this a navigation root

    IPossibleSite
      support local component registries
    """


class IChildSiteWillBeCreatedEvent(Interface):
    """An event that is fired before a child site is created
    """


class IChildSiteCreatedEvent(Interface):
    """An event that is fired after a child site is created
    """


class IChildSiteWillBeRemovedEvent(Interface):
    """An event that is fired before the child site is removed
    """


class IChildSiteRemovedEvent(Interface):
    """An event that is fired after a child site is removed
    """



# -*- coding: utf-8 -*-
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

try:
    from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
    HAS_ROBOT = True
except ImportError:
    HAS_ROBOT = False


class LineageLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import collective.lineage
        try:
            import plone.app.contenttypes
            self['has_pact'] = True
        except ImportError:
            self['has_pact'] = False
        self.loadZCML(package=collective.lineage)
        if self['has_pact']:
            self.loadZCML(package=plone.app.contenttypes)
        if self['has_pact']:
            z2.installProduct(app, 'plone.app.contenttypes')

    def setUpPloneSite(self, portal):
        if self['has_pact']:
            applyProfile(portal, 'plone.app.contenttypes:default')
        applyProfile(portal, 'collective.lineage:default')


LINEAGE_FIXTURE = LineageLayer()


LINEAGE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(LINEAGE_FIXTURE,),
    name='collective.lineage:IntegrationTesting'
)


LINEAGE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(LINEAGE_FIXTURE,),
    name='collective.lineage:FunctionalTesting'
)


if HAS_ROBOT:
    LINEAGE_ACCEPTANCE_TESTING = FunctionalTesting(
        bases=(
            LINEAGE_FIXTURE,
            REMOTE_LIBRARY_BUNDLE_FIXTURE,
            z2.ZSERVER_FIXTURE
        ),
        name='collective.lineage:AcceptanceTesting'
    )



# -*- coding: utf-8 -*-
from collective.lineage.testing import LINEAGE_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

try:
    # BBB
    import unittest2 as unittest
except ImportError:
    import unittest

PROJECTNAME = 'collective.lineage'


class UtilsTestCase(unittest.TestCase):

    layer = LINEAGE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Site Administrator'])

        self.portal.invokeFactory(id='childsite', type_name='Folder')
        self.childsite = self.portal['childsite']

    def test_enable(self):
        from collective.lineage.interfaces import IChildSite
        from collective.lineage.utils import enable_childsite
        self.assertFalse(IChildSite.providedBy(self.childsite))
        enable_childsite(self.childsite)
        self.assertTrue(IChildSite.providedBy(self.childsite))

    def test_disable(self):
        from collective.lineage.interfaces import IChildSite
        from collective.lineage.utils import disable_childsite
        from collective.lineage.utils import enable_childsite

        enable_childsite(self.childsite)
        self.assertTrue(IChildSite.providedBy(self.childsite))
        disable_childsite(self.childsite)
        self.assertFalse(IChildSite.providedBy(self.childsite))



# -*- coding: utf-8 -*-
from collective.lineage.interfaces import IChildSiteCreatedEvent
from collective.lineage.interfaces import IChildSiteRemovedEvent
from collective.lineage.interfaces import IChildSiteWillBeCreatedEvent
from collective.lineage.interfaces import IChildSiteWillBeRemovedEvent
from zope.component.interfaces import ObjectEvent
from zope.interface import implements


class ChildSiteWillBeCreatedEvent(ObjectEvent):
    implements(IChildSiteWillBeCreatedEvent)


class ChildSiteCreatedEvent(ObjectEvent):
    implements(IChildSiteCreatedEvent)


class ChildSiteWillBeRemovedEvent(ObjectEvent):
    implements(IChildSiteWillBeRemovedEvent)


class ChildSiteRemovedEvent(ObjectEvent):
    implements(IChildSiteRemovedEvent)



# -*- coding: utf-8 -*-
from logging import getLogger

logger = getLogger('collective.lineage.upgrades')


def upgradeToTwoZero(context):
    """ going from pre-2.X to 2.X needs to have the default profile run
        so that it can pick up the new actions
    """
    context.portal_setup.runAllImportStepsFromProfile(
        'profile-collective.lineage:default'
    )
    logger.info("Re-ran Lineage default profile to add 2.0 actions")



# -*- coding: utf-8 -*-
from Acquisition import aq_parent
from collective.lineage.interfaces import IChildSite
from collective.lineage.utils import disable_childsite
from collective.lineage.utils import enable_childsite
from plone.folder.interfaces import IFolder
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import isDefaultPage
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getMultiAdapter
from zope.i18nmessageid import MessageFactory


_ = MessageFactory('collective.lineage')


class LineageTool(BrowserView):

    def __init__(self, context, request):

        def _get_context(ctx, req):
            if not ctx:
                return None
            if isDefaultPage(ctx, req):
                return _get_context(aq_parent(ctx), req)
            return ctx

        # we don't want to enable/disable a childsite on a default page.
        # bend context to suitable parent.
        self.context = _get_context(context, request)
        self.request = request

    @property
    def available(self):
        """True, if the context can become a lineage subsite.
        """
        return IFolder.providedBy(self.context)

    @property
    def disabled(self):
        """True, if context is not a lineage subsite but could possibly be one.
        """
        return self.available and not self.enabled

    @property
    def enabled(self):
        """True, if context is a lineage subsite.
        """
        return IChildSite.providedBy(self.context)

    def enable(self):
        """Enable a lineage subsite on this context.
        """
        enable_childsite(self.context)

        # redirect
        self.request.response.redirect(self.context.absolute_url())

    def disable(self):
        """Disable a lineage subsite on this context.
        """
        disable_childsite(self.context)

        # redirect
        self.request.response.redirect(self.context.absolute_url())


class LineageSwitcherViewlet(BrowserView):
    render = ViewPageTemplateFile('switcher.pt')

    def __init__(self, context, request, view, manager):
        self.__parent__ = view
        self.context = context
        self.request = request
        self.view = view
        self.manager = manager

    def update(self):
        pass

    def sites(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        sites = [
            dict(title=c.Title, description=c.Description, url=c.getURL())
            for c in catalog(path='/',
                             object_provides=IChildSite.__identifier__,
                             sort_on='sortable_title')]
        if not sites:
            return []
        portal_url = getToolByName(self.context, 'portal_url')
        root_url = portal_url()
        if root_url not in [s['url'] for s in sites]:
            portal_object = portal_url.getPortalObject()
            sites.insert(0, dict(title=portal_object.Title(),
                                 description=portal_object.Description(),
                                 url=root_url))
        if len(sites) <= 1:
            return []
        return sites


class LineageUtils(BrowserView):

    def isChildSite(self):
        """Return ``True``, if the current context is within a childsite.
        """
        context = self.context
        request = self.request
        portal_state = getMultiAdapter(
            (context, request),
            name="plone_portal_state"
        )
        root_path = portal_state.navigation_root_path()
        nav_root = self.context.restrictedTraverse(root_path)
        return IChildSite.providedBy(nav_root)

    @property
    def current_childsite(self):
        """Return the current childsite.
        """
        childsite = None
        context = self.context
        request = self.request
        portal_state = getMultiAdapter(
            (context, request),
            name="plone_portal_state"
        )
        root_path = portal_state.navigation_root_path()
        nav_root = self.context.restrictedTraverse(root_path)
        if IChildSite.providedBy(nav_root):
            childsite = nav_root
        return childsite



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup


version = '2.1'
short_description = u"The microsite creation product for Plone"
long_description = u'\n\n'.join([
    open('README.rst').read(),
    open('CHANGES.rst').read()
])


setup(
    name='collective.lineage',
    version=version,
    description=short_description,
    long_description=long_description,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 5.0",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords='plone lineage',
    author='Six Feet Up, Inc.',
    author_email='info@sixfeetup.com',
    url='http://plone.org/products/collective-lineage',
    license='GPL',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['collective'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'Plone',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zope.component import queryUtility
from plone.app.imaging.interfaces import IImagingSchema


def get_image_traverser(context):
    siteroot = queryUtility(IPloneSiteRoot)
    return IImagingSchema(siteroot)



# -*- coding: utf-8 -*-
from collective.lineage.testing import LINEAGE_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import getGlobalSiteManager

try:
    # BBB
    import unittest2 as unittest
except ImportError:
    import unittest

PROJECTNAME = 'collective.lineage'


class AdapterTestCase(unittest.TestCase):

    layer = LINEAGE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Site Administrator'])
        self.portal.invokeFactory('Folder', 'site1')

    def test_adapter_registered_correctly(self):
        """this test is ATCT only"""
        if self.layer['has_pact']:
            return
        from plone.app.imaging.interfaces import IImagingSchema

        sm = getGlobalSiteManager()
        registrations = [a for a in sm.registeredAdapters()
                         if a.provided == IImagingSchema]
        self.assertEqual(len(registrations), 1)

    def test_childsite_is_image_traverser(self):
        """this test is ATCT only"""
        if self.layer['has_pact']:
            return
        from plone.app.imaging.interfaces import IImagingSchema
        from plone.app.imaging.configlet import ImagingControlPanelAdapter
        from collective.lineage.utils import enable_childsite
        enable_childsite(self.portal.site1)
        configuration = IImagingSchema(self.portal.site1)
        self.assertIsInstance(configuration, ImagingControlPanelAdapter)
